# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from endex_factset_api import EndexFactsetAPI, AsyncEndexFactsetAPI
from endex_factset_api.types import MetricsResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestMetrics:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_list(self, client: EndexFactsetAPI) -> None:
        metric = client.metrics.list()
        assert_matches_type(MetricsResponse, metric, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: EndexFactsetAPI) -> None:
        metric = client.metrics.list(
            category="INCOME_STATEMENT",
            metric_data_type="metricDataType",
            subcategory="ASSETS",
        )
        assert_matches_type(MetricsResponse, metric, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: EndexFactsetAPI) -> None:
        response = client.metrics.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        metric = response.parse()
        assert_matches_type(MetricsResponse, metric, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: EndexFactsetAPI) -> None:
        with client.metrics.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            metric = response.parse()
            assert_matches_type(MetricsResponse, metric, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncMetrics:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_list(self, async_client: AsyncEndexFactsetAPI) -> None:
        metric = await async_client.metrics.list()
        assert_matches_type(MetricsResponse, metric, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncEndexFactsetAPI) -> None:
        metric = await async_client.metrics.list(
            category="INCOME_STATEMENT",
            metric_data_type="metricDataType",
            subcategory="ASSETS",
        )
        assert_matches_type(MetricsResponse, metric, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncEndexFactsetAPI) -> None:
        response = await async_client.metrics.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        metric = await response.parse()
        assert_matches_type(MetricsResponse, metric, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncEndexFactsetAPI) -> None:
        async with async_client.metrics.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            metric = await response.parse()
            assert_matches_type(MetricsResponse, metric, path=["response"])

        assert cast(Any, response.is_closed) is True
